<?php

namespace Bydedi\Shipping\Chronopost;

use SiteSQL;
use Shopbydedi\Shipping\RelayInterface;
use Shopbydedi\Shipping\Relay;

/**
 * Class Chronopost
 * @package Bydedi\Shipping\Chronopost
 */
class Chronopost implements RelayInterface
{
    /**
     * @var SiteSQL
     */
    protected $sql;

    /**
     * @var array
     */
    protected $endpoint;

    /**
     * @var string
     */
    protected $accountNumber;

    /**
     * @var string
     */
    protected $password;

    /**
     * Chronopost constructor.
     * @param $config
     */
    public function __construct($config)
    {
        $this->sql = SiteSQL::getInstance();

        $this->endpoint = 'https://ws.chronopost.fr';

        $this->accountNumber = $config['chronopost']['account_number'];
        $this->password = $config['chronopost']['password'];
    }

    /**
     * @param array $formvars
     *
     * @return array
     *
     * @throws \DediException
     */
    public function getList($formvars = array())
    {
        $param = [
            'address' => '',
            'zipCode' => $formvars['code_postal'],
            'city' => $formvars['ville'],
            'countryCode' => $formvars['pays'],
            'type' => 'P',
            'productCode' => 1,
            'service' => 'L',
            'weight' => '',
            'shippingDate' => date('d/m/Y'),
            'maxPointChronopost' => 9,
            'maxDistanceSearch' => 50,
            'holidayTolerant' => 1,
            'language' => 'FR',
        ];

        // remove empty entries
        $param = array_filter($param);

        $result = $this->call('recherchePointChronopost', $param);

        if ($result->return->errorCode != 0) {
            throw new \DediException(
                sprintf('Error code %s - %s', $result->return->errorCode, $result->return->errorMessage)
            );
        }

        $relais = [];
        foreach ($result->return->listePointRelais as $rel) {
            $relais[] = $this->formatRelay((array) $rel);
        }

        return $relais;
    }

    /**
     * @param string $relayId
     * @param string $countryIso
     *
     * @return Relay
     *
     * @throws \DediException
     */
    public function getRelay($relayId, $countryIso = '')
    {
        $param = [
            'identifiant' => $relayId,
            'countryCode' => $countryIso,
            'language' => 'FR',
        ];

        $result = $this->call('rechercheDetailPointChronopostInter', $param);

        if ($result->return->errorCode != 0) {
            throw new \DediException(
                sprintf('Error code %s - %s', $result->return->errorCode, $result->return->errorMessage)
            );
        }

        return $this->formatRelay((array) $result->return->listePointRelais);
    }

    /**
     * @param array $data
     * @return Relay
     */
    public function formatRelay(array $data)
    {

        $relay = new Relay();
        $relay->id = trim($data['identifiant']);
        $relay->name = trim($data['nom']);
        $relay->addr1 = trim($data['adresse1']);
        $relay->addr2 = (!empty($data['adresse2'])) ? trim($data['adresse2']) : '';
        $relay->zipcode = trim($data['codePostal']);
        $relay->city = trim($data['localite']);
        $relay->country = trim($data['codePays']);
        $relay->latitude = $data['coordGeolocalisationLatitude'];
        $relay->longitude = $data['coordGeolocalisationLongitude'];
        $relay->distance = $data['distanceEnMetre'];

        // TODO : revoir horaires
        $relay->horaires = $data['listeHoraireOuverture'];

        return $relay;
    }

    /**
     * Make an API call
     *
     * @param $func
     * @param $param
     *
     * @return mixed
     */
    protected function call($func, $param)
    {
        $auth = [
            'accountNumber' => $this->accountNumber,
            'password' => $this->password,
        ];

        $param = $auth + $param;

        $soap = $this->getSoapFor($func);
        return $soap->$func($param);
    }

    /**
     * @param $func
     * @return \SoapClient
     */
    protected function getSoapFor($func)
    {
        switch ($func) {
            case 'recherchePointChronopost':
            case 'rechercheDetailPointChronopostInter':
                return new \SoapClient($this->endpoint . '/recherchebt-ws-cxf/PointRelaisServiceWS?wsdl');
        }
    }
}
